// C&C NLP tools
// Copyright (c) Universities of Edinburgh, Oxford and Sydney
// Copyright (c) James R. Curran
//
// This software is covered by a non-commercial use licence.
// See LICENCE.txt for the full text of the licence.
//
// If LICENCE.txt is not included in this distribution
// please email candc@it.usyd.edu.au to obtain a copy.

#include "base.h"

using namespace std;

namespace NLP {

  const string PREFACE_HEADER = "# this file was generated by the following command(s):";

  string
  read_preface(const string &uri, istream &in, ulong &nlines, bool mandatory){
    char c = in.peek();
    if(c != '#'){
      if(mandatory)
	throw NLP::IOException("the stream does not start with the mandatory preface", uri);
      return "";
    }
    
    string res;
    string line;
    while(getline(in, line)){
      ++nlines;
      if(line.length() == 0)
	break;
      
      if(line[0] != '#')
	throw NLP::IOException("uncommented line inside comment preface", uri);
      
      if(line.find(PREFACE_HEADER) == 0)
	continue;
      
      res += line;
      res += '\n';
    }
    
    return res;
  }

  string
  start_preface(int argc, char **argv){
    string res = PREFACE_HEADER;
    res += "\n#   ";
    res += argv[0];
    for(int i = 1; i < argc; ++i){
      res += ' ';
      res += argv[i];
    }
    res += '\n';
    return res;
  }
  
}
